@@ -63,7 +63,8 @@ def create_branch(args, patches):
     except CalledProcessError as err:
         logging.debug("Branch %s does not exist" % args.name)
     subprocess.check_call(["git", "checkout", "-b", args.name])
-    subprocess.check_call(["git", "fetch", "upstream"])
+    if args.fetch:
+        subprocess.check_call(["git", "fetch", "upstream"])
     subprocess.check_call(["git", "rebase", args.remote])
     logging.info("Create and rebased branch %s" % args.name)
 
@@ -132,8 +133,8 @@ def patch_apply(args, patches):
     for section in patches["sections"]:
         d = vars(args)
         d['section'] = section
-        section_apply(args, patches)
-
+        if section_apply(args, patches):
+            args.from_patch = None
 
 def patch_commits(args, patches):
     print "\n".join(args.section["commits"])
@@ -153,17 +154,39 @@ def _generate_edit_metadata(commit, subject):
         outfile.write(yaml.dump(metadata, default_flow_style=False))
     logging.info("Created metadata_edit file for %s" % commit)
 
-
+"""
+" If from_patch is set in args, then no patch will be applied until 
+" the patch named by from_patch is found
+"
+" Returns True if either
+"   1. Patch passed through --from-patch is found in this section
+"   2. If --from-patch was not set
+"""
 def section_apply(args, patches):
     subject_re = re.compile("Subject: \[PATCH.*\] (.*)^\-\-\-\n", re.S | re.M)
+    algo = "-3"
+    if args.reject:
+        algo = "--reject"
+
+    patch_required = False
+    if args.from_patch is None:
+        patch_required = True
+
     for commit in args.section["commits"]:
+        if not patch_required:
+            if commit == os.path.basename(args.from_patch):
+                patch_required = True
+                # from_patch is excluded, fall through to continue
+            logging.info("Skipping: %s " % commit)
+            continue
+
         logging.info("Processing: %s " % commit)
         with open('.patch/%s' % commit, 'r') as commit_file:
             data = commit_file.read()
         subject = subject_re.search(data).group(1)
         logging.info(subject)
         try:
-            subprocess.check_call(["git", "apply", "-3", ".patch/%s" % commit])
+            subprocess.check_call(["git", "apply", algo, ".patch/%s" % commit])
             if args.patch is not None and commit == os.path.basename(args.patch):
                 _generate_edit_metadata(commit, subject)
                 raise UserAbortedException("User aborted at %s " % commit)
@@ -171,7 +194,10 @@ def section_apply(args, patches):
             logging.error("%s failed to apply" % commit)
             _generate_edit_metadata(commit, subject)
             raise err
+        if args.reject:
+            subprocess.check_call(["git", "add", "."])
         subprocess.check_call(["git", "commit", "-m", "[Patch] %s" % subject])
+    return patch_required
 
 
 def squash(args, patches):
@@ -239,7 +265,10 @@ def main():
     branch_parser = sub_parsers.add_parser("create-branch", help="Create a new development branch")
     branch_parser.add_argument("-n", "--name", help="Name of branch", default="git_patch")
     branch_parser.add_argument("-r", "--remote", help="Name of Git Remote", default="upstream/master")
-    branch_parser.set_defaults(func=create_branch)
+    fetch_parser = branch_parser.add_mutually_exclusive_group(required=False)
+    fetch_parser.add_argument("--no-fetch", dest='fetch', help="Do not fetch from upstream", action='store_false')
+    fetch_parser.add_argument("--fetch", dest='fetch', help="Fetch from upstream", action='store_true')
+    branch_parser.set_defaults(func=create_branch, fetch=True)
 
     list_parser = sub_parsers.add_parser("list", help="List all patches")
     list_parser.set_defaults(func=list_patches)
@@ -258,6 +287,9 @@ def main():
 
     patch_apply_parser = sub_parsers.add_parser("apply", help="Apply all patch")
     patch_apply_parser.add_argument("-p", "--patch", help="File name of patch to stop")
+    patch_apply_parser.add_argument("--from-patch", dest='from_patch', help="Applies patches after the given file")
+    patch_apply_parser.add_argument("--reject", help="Patches are applied using 3-way. Use reject instead",
+                                    action='store_true')
     patch_apply_parser.set_defaults(func=patch_apply)
 
     fix_parser = sub_parsers.add_parser("fix-patch", help="Begin editing a patch")
@@ -280,6 +312,9 @@ def main():
 
             sub_apply = sub_sub.add_parser("apply", help="Apply a patch")
             sub_apply.add_argument("-p", "--patch", help="File name of patch to stop")
+            sub_apply.add_argument("--from-patch", dest='from_patch', help="Applies patches after the given file")
+            sub_apply.add_argument("--reject", help="Patches are applied using 3-way. Use reject instead",
+                                   action='store_true')
             sub_apply.set_defaults(func=section_apply, section=s)
 
             squash_parser = sub_sub.add_parser("squash", help="Begin editing a patch")
